"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("../meta/node");
const buildDescriptionTree_1 = require("./buildDescriptionTree");
const nodeDescription_1 = require("./interfaces/nodeDescription");
const nodeType_1 = require("./interfaces/nodeType");
const nodeDescriptionToLeaf_1 = require("./nodeDescriptionToLeaf");
const typeDependentDictionary_1 = require("./typeDependentDictionary");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function buildMetaTree(tree, theme, settings) {
    const nodeDescriptionTree = buildDescriptionTree_1.buildDescriptionTree(undefined, tree, settings);
    return nodeDescriptionTreeToMetaTreeRecursive(nodeDescriptionTree, theme, settings);
}
exports.buildMetaTree = buildMetaTree;
function getNodePrebuildedTheme(nodeDescription, theme) {
    return typeDependentDictionary_1.getFromTypeDependentDictionary(theme, nodeDescription.typeTuple);
}
function nodeDescriptionTreeToMetaTreeRecursive(nodeDescription, theme, settings) {
    const nodePrebuildedTheme = getNodePrebuildedTheme(nodeDescription, theme);
    const leaf = nodeDescriptionToLeaf_1.nodeDescriptionToLeaf(nodeDescription, nodePrebuildedTheme);
    const buildedMetaNode = new node_1.MetaNode(leaf, nodePrebuildedTheme.arm.armGenerator, nodePrebuildedTheme.arm.armWidthGenerator);
    if (nodeDescription_1.guardNodeDescription(nodeType_1.NodeBroadTypeEnum.Enumerable, nodeDescription) ||
        nodeDescription_1.guardNodeDescription(nodeType_1.NodeBroadTypeEnum.Iterable, nodeDescription)) {
        buildedMetaNode.children = nodeDescription.subEntries.map((childDescription) => {
            return nodeDescriptionTreeToMetaTreeRecursive(childDescription, theme, settings);
        });
    }
    return buildedMetaNode;
}
//# sourceMappingURL=buildMetaTree.js.map