"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const nodeDescription_1 = require("./interfaces/nodeDescription");
const textContainer_1 = require("../text/textContainer");
const strictUnicode_1 = require("../text/strictUnicode");
const nodeType_1 = require("./interfaces/nodeType");
function nodeDescriptionToLeaf(nodeDescription, nodeTheme) {
    const preparedPack = prepareTextToPack(nodeDescription, nodeTheme);
    const leaf = preparedPackToLeaf(preparedPack, nodeTheme.style);
    return leaf;
}
exports.nodeDescriptionToLeaf = nodeDescriptionToLeaf;
function prepareTextToPack(nodeDescription, nodeTheme) {
    var _a, _b, _c, _d;
    const preparedPack = {};
    // icon
    if (nodeTheme.icon) {
        const preparedPackItem = {
            wholeItemWrapperStyle: nodeTheme.style,
        };
        if (nodeTheme.icon.predelimiter && nodeTheme.icon.predelimiter.container) {
            preparedPackItem.predelimiter = nodeTheme.icon.predelimiter.container;
        }
        if (nodeTheme.icon.prefix && nodeTheme.icon.prefix.container) {
            preparedPackItem.prefix = nodeTheme.icon.prefix.container;
        }
        if (nodeTheme.icon.content && nodeTheme.icon.content.container) {
            preparedPackItem.content = nodeTheme.icon.content.container;
        }
        if (nodeTheme.icon.postfix && nodeTheme.icon.postfix.container) {
            preparedPackItem.postfix = nodeTheme.icon.postfix.container;
        }
        if (nodeTheme.icon.postdelimiter && nodeTheme.icon.postdelimiter.container) {
            preparedPackItem.postdelimiter = nodeTheme.icon.postdelimiter.container;
        }
        preparedPackItem.wholeItemWrapperStyle = nodeTheme.icon.style,
            preparedPack.icon = preparedPackItem;
    }
    // key
    if (nodeDescription.key !== undefined) {
        if (nodeTheme.key) {
            const preparedPackItem = {
                wholeItemWrapperStyle: nodeTheme.style,
            };
            if (nodeTheme.key.predelimiter && nodeTheme.key.predelimiter.container) {
                preparedPackItem.predelimiter = nodeTheme.key.predelimiter.container;
            }
            if (nodeTheme.key.prefix && nodeTheme.key.prefix.container) {
                preparedPackItem.prefix = nodeTheme.key.prefix.container;
            }
            {
                const nodeKeyString = (nodeDescription.key.toString) ? nodeDescription.key.toString() : String(nodeDescription.key);
                preparedPackItem.content = new textContainer_1.AtomicTextContainer(new strictUnicode_1.StrictUnicodeLine(nodeKeyString), (_a = nodeTheme.key.content) === null || _a === void 0 ? void 0 : _a.style);
            }
            if (nodeTheme.key.postfix && nodeTheme.key.postfix.container) {
                preparedPackItem.postfix = nodeTheme.key.postfix.container;
            }
            if (nodeTheme.key.postdelimiter && nodeTheme.key.postdelimiter.container) {
                preparedPackItem.postdelimiter = nodeTheme.key.postdelimiter.container;
            }
            preparedPackItem.wholeItemWrapperStyle = nodeTheme.key.style,
                preparedPack.key = preparedPackItem;
        }
    }
    // value
    if (!nodeDescription_1.guardNodeDescription(nodeType_1.NodeBroadTypeEnum.Dead, nodeDescription)) {
        if (nodeDescription.value !== undefined) {
            if (nodeTheme.value) {
                const preparedPackItem = {
                    wholeItemWrapperStyle: nodeTheme.style,
                };
                if (nodeTheme.value.predelimiter && nodeTheme.value.predelimiter.container) {
                    preparedPackItem.predelimiter = nodeTheme.value.predelimiter.container;
                }
                if (nodeTheme.value.prefix && nodeTheme.value.prefix.container) {
                    preparedPackItem.prefix = nodeTheme.value.prefix.container;
                }
                {
                    const nodeKeyString = (nodeDescription.value.toString) ? nodeDescription.value.toString() : String(nodeDescription.value);
                    preparedPackItem.content = new textContainer_1.AtomicTextContainer(new strictUnicode_1.StrictUnicodeLine(nodeKeyString), (_b = nodeTheme.value.content) === null || _b === void 0 ? void 0 : _b.style);
                }
                if (nodeTheme.value.postfix && nodeTheme.value.postfix.container) {
                    preparedPackItem.postfix = nodeTheme.value.postfix.container;
                }
                if (nodeTheme.value.postdelimiter && nodeTheme.value.postdelimiter.container) {
                    preparedPackItem.postdelimiter = nodeTheme.value.postdelimiter.container;
                }
                preparedPackItem.wholeItemWrapperStyle = nodeTheme.value.style,
                    preparedPack.value = preparedPackItem;
            }
        }
    }
    // info
    if (nodeDescription.info !== undefined) {
        if (nodeTheme.info) {
            const preparedPackItem = {
                wholeItemWrapperStyle: nodeTheme.style,
            };
            if (nodeTheme.info.predelimiter && nodeTheme.info.predelimiter.container) {
                preparedPackItem.predelimiter = nodeTheme.info.predelimiter.container;
            }
            if (nodeTheme.info.prefix && nodeTheme.info.prefix.container) {
                preparedPackItem.prefix = nodeTheme.info.prefix.container;
            }
            {
                const nodeKeyString = (nodeDescription.info.toString) ? nodeDescription.info.toString() : String(nodeDescription.info);
                preparedPackItem.content = new textContainer_1.AtomicTextContainer(new strictUnicode_1.StrictUnicodeLine(nodeKeyString), (_c = nodeTheme.info.content) === null || _c === void 0 ? void 0 : _c.style);
            }
            if (nodeTheme.info.postfix && nodeTheme.info.postfix.container) {
                preparedPackItem.postfix = nodeTheme.info.postfix.container;
            }
            if (nodeTheme.info.postdelimiter && nodeTheme.info.postdelimiter.container) {
                preparedPackItem.postdelimiter = nodeTheme.info.postdelimiter.container;
            }
            preparedPackItem.wholeItemWrapperStyle = nodeTheme.info.style,
                preparedPack.info = preparedPackItem;
        }
    }
    // remark
    if (nodeDescription.remark !== undefined) {
        if (nodeTheme.remark) {
            const preparedPackItem = {
                wholeItemWrapperStyle: nodeTheme.style,
            };
            if (nodeTheme.remark.predelimiter && nodeTheme.remark.predelimiter.container) {
                preparedPackItem.predelimiter = nodeTheme.remark.predelimiter.container;
            }
            if (nodeTheme.remark.prefix && nodeTheme.remark.prefix.container) {
                preparedPackItem.prefix = nodeTheme.remark.prefix.container;
            }
            {
                const nodeKeyString = (nodeDescription.remark.toString) ? nodeDescription.remark.toString() : String(nodeDescription.remark);
                preparedPackItem.content = new textContainer_1.AtomicTextContainer(new strictUnicode_1.StrictUnicodeLine(nodeKeyString), (_d = nodeTheme.remark.content) === null || _d === void 0 ? void 0 : _d.style);
            }
            if (nodeTheme.remark.postfix && nodeTheme.remark.postfix.container) {
                preparedPackItem.postfix = nodeTheme.remark.postfix.container;
            }
            if (nodeTheme.remark.postdelimiter && nodeTheme.remark.postdelimiter.container) {
                preparedPackItem.postdelimiter = nodeTheme.remark.postdelimiter.container;
            }
            preparedPackItem.wholeItemWrapperStyle = nodeTheme.remark.style,
                preparedPack.remark = preparedPackItem;
        }
    }
    return preparedPack;
}
function preparedPackToLeaf(preparedPack, wholeLeafStyle) {
    // render line parts to leaf with delimeters when needed
    const leafChildren = [];
    let isDelimiterNeed = false;
    const unorederedPreparedItems = Object.values(preparedPack); // TODO: custom order
    for (let itemId = 0; itemId < unorederedPreparedItems.length; itemId++) {
        const preparedPackItem = unorederedPreparedItems[itemId];
        const isFirst = itemId === 0;
        const isLast = itemId === unorederedPreparedItems.length - 1;
        // in fact i really need a isLast, because isFirst is skipped anyway
        // on first run because of isDelimiterNeed = false;
        if (preparedPackItem) {
            const itemChildren = [];
            if (preparedPackItem.predelimiter && isDelimiterNeed && !isFirst) {
                itemChildren.push(preparedPackItem.predelimiter);
                isDelimiterNeed = false;
            }
            if (preparedPackItem.prefix) {
                itemChildren.push(preparedPackItem.prefix);
                isDelimiterNeed = true;
            }
            if (preparedPackItem.content) {
                itemChildren.push(preparedPackItem.content);
                isDelimiterNeed = true;
            }
            if (preparedPackItem.postfix) {
                itemChildren.push(preparedPackItem.postfix);
                isDelimiterNeed = true;
            }
            if (preparedPackItem.postdelimiter && isDelimiterNeed && !isLast) {
                itemChildren.push(preparedPackItem.postdelimiter);
                isDelimiterNeed = false;
            }
            if (itemChildren.length > 0) {
                const itemWrapper = new textContainer_1.NonatomicTextContainer(itemChildren, preparedPackItem.wholeItemWrapperStyle);
                leafChildren.push(itemWrapper);
            }
        }
    }
    return new textContainer_1.NonatomicTextContainer(leafChildren, wholeLeafStyle);
}
//# sourceMappingURL=nodeDescriptionToLeaf.js.map