"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const style_1 = require("../../../text/style");
const styleTypes_1 = require("../../../text/styleTypes");
const standartPalette = {
    color: {
        info: { r: 96, g: 80, b: 96 },
        remark: { r: 255, g: 128, b: 0 },
        value: { r: 225, g: 225, b: 225 },
        unvalue: { r: 180, g: 180, b: 120 },
        enumerable: {
            arm: { r: 60, g: 70, b: 60 },
            icon: { r: 55, g: 105, b: 55 },
            iconAccent: { r: 100, g: 200, b: 100 },
            key: { r: 100, g: 200, b: 100 },
            keyDelimiter: { r: 60, g: 70, b: 60 },
        },
        iterable: {
            arm: { r: 60, g: 60, b: 70 },
            icon: { r: 55, g: 55, b: 105 },
            iconAccent: { r: 100, g: 100, b: 200 },
            key: { r: 100, g: 100, b: 200 },
            keyDelimiter: { r: 60, g: 60, b: 70 },
        },
        single: {
            arm: { r: 60, g: 60, b: 60 },
            icon: { r: 60, g: 60, b: 60 },
            iconAccent: { r: 150, g: 150, b: 150 },
            key: { r: 100, g: 100, b: 100 },
            keyDelimiter: { r: 60, g: 60, b: 60 },
        },
        dead: {
            arm: { r: 70, g: 60, b: 60 },
            icon: { r: 200, g: 100, b: 100 },
            iconAccent: { r: 105, g: 55, b: 55 },
            key: { r: 200, g: 100, b: 100 },
            keyDelimiter: { r: 70, g: 60, b: 60 },
            elipsis: { r: 80, g: 80, b: 80 },
        },
    },
};
const safeAnsiPalette = {
    color: {
        info: { r: 128, g: 64, b: 128 },
        remark: { r: 255, g: 128, b: 0 },
        value: { r: 192, g: 192, b: 192 },
        unvalue: { r: 192, g: 192, b: 128 },
        enumerable: {
            arm: { r: 0, g: 128, b: 0 },
            icon: { r: 64, g: 128, b: 64 },
            iconAccent: { r: 64, g: 255, b: 64 },
            key: { r: 64, g: 255, b: 64 },
            keyDelimiter: { r: 0, g: 64, b: 0 },
        },
        iterable: {
            arm: { r: 0, g: 0, b: 128 },
            icon: { r: 64, g: 64, b: 128 },
            iconAccent: { r: 64, g: 64, b: 255 },
            key: { r: 64, g: 64, b: 255 },
            keyDelimiter: { r: 0, g: 0, b: 64 },
        },
        single: {
            arm: { r: 64, g: 64, b: 64 },
            icon: { r: 64, g: 64, b: 64 },
            iconAccent: { r: 192, g: 192, b: 192 },
            key: { r: 128, g: 128, b: 128 },
            keyDelimiter: { r: 64, g: 64, b: 64 },
        },
        dead: {
            arm: { r: 128, g: 0, b: 0 },
            icon: { r: 128, g: 64, b: 64 },
            iconAccent: { r: 255, g: 64, b: 64 },
            key: { r: 255, g: 64, b: 64 },
            keyDelimiter: { r: 64, g: 0, b: 0 },
            elipsis: { r: 64, g: 64, b: 64 },
        },
    },
};
const richFontsPalette = Object.assign({}, standartPalette, {
    styleSwatches: {
        key: new style_1.StyleSwitches({ [styleTypes_1.StyleSwitchesEnum.Bold]: false }),
        info: new style_1.StyleSwitches({ [styleTypes_1.StyleSwitchesEnum.Italic]: true }),
    }
});
function styleIfNeed(foreground, background, styleSwatches) {
    if (foreground || background || styleSwatches) {
        return new style_1.Style(foreground, background, styleSwatches);
    }
    else {
        return undefined;
    }
}
function buildTheme(palette) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35;
    const theme = {
        global: {
            visibility: true,
            icon: {
                postdelimiter: {
                    line: ' ',
                }
            },
            key: {
                postdelimiter: {
                    line: ': ',
                }
            },
            value: {
                style: styleIfNeed((_a = palette.color) === null || _a === void 0 ? void 0 : _a.value),
            },
            info: {
                predelimiter: {
                    line: ' · ',
                },
                style: styleIfNeed((_b = palette.color) === null || _b === void 0 ? void 0 : _b.info, undefined, (_c = palette.styleSwatches) === null || _c === void 0 ? void 0 : _c.info),
            },
            remark: {
                predelimiter: {
                    line: ' * ',
                },
                style: styleIfNeed((_d = palette.color) === null || _d === void 0 ? void 0 : _d.remark),
            },
        },
        broad: {
            enumerable: {
                arm: {
                    commonChars: ['├─╴', '│  ', '╰─╴',],
                    width: 3,
                    style: styleIfNeed((_f = (_e = palette.color) === null || _e === void 0 ? void 0 : _e.enumerable) === null || _f === void 0 ? void 0 : _f.arm),
                },
                icon: {
                    prefix: {
                        line: '{',
                    },
                    content: {
                        line: ' ',
                        style: styleIfNeed((_h = (_g = palette.color) === null || _g === void 0 ? void 0 : _g.enumerable) === null || _h === void 0 ? void 0 : _h.iconAccent),
                    },
                    postfix: {
                        line: '}',
                    },
                    style: styleIfNeed((_k = (_j = palette.color) === null || _j === void 0 ? void 0 : _j.enumerable) === null || _k === void 0 ? void 0 : _k.icon),
                },
                key: {
                    style: styleIfNeed((_m = (_l = palette.color) === null || _l === void 0 ? void 0 : _l.enumerable) === null || _m === void 0 ? void 0 : _m.key, undefined, (_o = palette.styleSwatches) === null || _o === void 0 ? void 0 : _o.key),
                    postdelimiter: {
                        style: styleIfNeed((_q = (_p = palette.color) === null || _p === void 0 ? void 0 : _p.enumerable) === null || _q === void 0 ? void 0 : _q.keyDelimiter),
                    }
                },
            },
            iterable: {
                arm: {
                    commonChars: ['├─╸', '│  ', '└─╸',],
                    width: 3,
                    style: styleIfNeed((_s = (_r = palette.color) === null || _r === void 0 ? void 0 : _r.iterable) === null || _s === void 0 ? void 0 : _s.arm),
                },
                icon: {
                    prefix: {
                        line: '[',
                    },
                    content: {
                        line: ' ',
                        style: styleIfNeed((_u = (_t = palette.color) === null || _t === void 0 ? void 0 : _t.iterable) === null || _u === void 0 ? void 0 : _u.iconAccent),
                    },
                    postfix: {
                        line: ']',
                    },
                    style: styleIfNeed((_w = (_v = palette.color) === null || _v === void 0 ? void 0 : _v.iterable) === null || _w === void 0 ? void 0 : _w.icon),
                },
                key: {
                    style: styleIfNeed((_y = (_x = palette.color) === null || _x === void 0 ? void 0 : _x.iterable) === null || _y === void 0 ? void 0 : _y.key, undefined, (_z = palette.styleSwatches) === null || _z === void 0 ? void 0 : _z.key),
                    postdelimiter: {
                        style: styleIfNeed((_1 = (_0 = palette.color) === null || _0 === void 0 ? void 0 : _0.iterable) === null || _1 === void 0 ? void 0 : _1.keyDelimiter),
                    }
                },
            },
            single: {
                arm: {
                    commonChars: ['├─╴', '│  ', '└─╴',],
                    width: 3,
                    style: styleIfNeed((_3 = (_2 = palette.color) === null || _2 === void 0 ? void 0 : _2.single) === null || _3 === void 0 ? void 0 : _3.arm),
                },
                icon: {
                    prefix: {
                        line: '<',
                    },
                    content: {
                        line: ' ',
                        style: styleIfNeed((_5 = (_4 = palette.color) === null || _4 === void 0 ? void 0 : _4.single) === null || _5 === void 0 ? void 0 : _5.iconAccent),
                    },
                    postfix: {
                        line: '>',
                    },
                    style: styleIfNeed((_7 = (_6 = palette.color) === null || _6 === void 0 ? void 0 : _6.single) === null || _7 === void 0 ? void 0 : _7.icon),
                },
                key: {
                    style: styleIfNeed((_9 = (_8 = palette.color) === null || _8 === void 0 ? void 0 : _8.single) === null || _9 === void 0 ? void 0 : _9.key, undefined, (_10 = palette.styleSwatches) === null || _10 === void 0 ? void 0 : _10.key),
                    postdelimiter: {
                        style: styleIfNeed((_12 = (_11 = palette.color) === null || _11 === void 0 ? void 0 : _11.single) === null || _12 === void 0 ? void 0 : _12.keyDelimiter),
                    }
                },
            },
            dead: {
                arm: {
                    commonChars: ['├─╴', '│  ', '╰─╴',],
                    width: 3,
                    style: styleIfNeed((_14 = (_13 = palette.color) === null || _13 === void 0 ? void 0 : _13.dead) === null || _14 === void 0 ? void 0 : _14.arm),
                },
                icon: {
                    prefix: {
                        line: '>',
                    },
                    content: {
                        line: ' ',
                        style: styleIfNeed((_16 = (_15 = palette.color) === null || _15 === void 0 ? void 0 : _15.dead) === null || _16 === void 0 ? void 0 : _16.iconAccent),
                    },
                    postfix: {
                        line: '<',
                    },
                    style: styleIfNeed((_18 = (_17 = palette.color) === null || _17 === void 0 ? void 0 : _17.dead) === null || _18 === void 0 ? void 0 : _18.icon),
                },
                key: {
                    style: styleIfNeed((_20 = (_19 = palette.color) === null || _19 === void 0 ? void 0 : _19.dead) === null || _20 === void 0 ? void 0 : _20.key, undefined, (_21 = palette.styleSwatches) === null || _21 === void 0 ? void 0 : _21.key),
                    postdelimiter: {
                        style: styleIfNeed((_23 = (_22 = palette.color) === null || _22 === void 0 ? void 0 : _22.dead) === null || _23 === void 0 ? void 0 : _23.keyDelimiter),
                    }
                },
            },
        },
        fine: {
            dead: {
                circularReference: {
                    icon: {
                        content: {
                            line: '∞',
                        }
                    }
                },
                elipsis: {
                    icon: {
                        prefix: {
                            line: '',
                        },
                        content: {
                            line: '...',
                        },
                        postfix: {
                            line: '',
                        },
                        style: styleIfNeed((_25 = (_24 = palette.color) === null || _24 === void 0 ? void 0 : _24.dead) === null || _25 === void 0 ? void 0 : _25.elipsis),
                    }
                },
            },
            single: {
                bigInt: {
                    icon: {
                        content: {
                            line: 'I',
                        }
                    }
                },
                boolean: {
                    icon: {
                        content: {
                            line: 'B',
                        }
                    }
                },
                booleanObject: {
                    icon: {
                        content: {
                            line: 'B',
                        }
                    }
                },
                date: {
                    icon: {
                        content: {
                            line: 'D',
                        }
                    }
                },
                function: {
                    icon: {
                        content: {
                            line: 'F',
                        }
                    }
                },
                null: {
                    icon: {
                        content: {
                            line: 'X',
                        }
                    },
                    value: {
                        style: styleIfNeed((_26 = palette.color) === null || _26 === void 0 ? void 0 : _26.unvalue)
                    }
                },
                number: {
                    icon: {
                        content: {
                            line: 'N',
                        }
                    }
                },
                numberObject: {
                    icon: {
                        content: {
                            line: 'N',
                        }
                    }
                },
                string: {
                    icon: {
                        content: {
                            line: 'T',
                        }
                    }
                },
                stringObject: {
                    icon: {
                        content: {
                            line: 'T',
                        }
                    }
                },
                symbol: {
                    icon: {
                        content: {
                            line: 'S',
                        }
                    }
                },
                undefined: {
                    icon: {
                        content: {
                            line: ' ',
                        }
                    },
                    value: {
                        style: styleIfNeed((_27 = palette.color) === null || _27 === void 0 ? void 0 : _27.unvalue)
                    }
                },
                unknown: {
                    icon: {
                        content: {
                            line: '?',
                        }
                    }
                },
                weakMap: {
                    icon: {
                        content: {
                            line: 'M',
                        }
                    }
                },
                weakSet: {
                    icon: {
                        content: {
                            line: 'S',
                        }
                    }
                },
            },
            enumerable: {
                map: {
                    icon: {
                        content: {
                            line: 'M',
                        }
                    }
                },
                object: {
                    icon: {
                        content: {
                            line: ' ',
                            style: styleIfNeed((_29 = (_28 = palette.color) === null || _28 === void 0 ? void 0 : _28.enumerable) === null || _29 === void 0 ? void 0 : _29.icon),
                        },
                        style: styleIfNeed((_31 = (_30 = palette.color) === null || _30 === void 0 ? void 0 : _30.enumerable) === null || _31 === void 0 ? void 0 : _31.iconAccent),
                    }
                },
                error: {
                    icon: {
                        content: {
                            line: 'E',
                        }
                    }
                },
                unknown: {
                    icon: {
                        content: {
                            line: '?',
                        }
                    }
                },
            },
            iterable: {
                array: {
                    icon: {
                        content: {
                            line: ' ',
                            style: styleIfNeed((_33 = (_32 = palette.color) === null || _32 === void 0 ? void 0 : _32.iterable) === null || _33 === void 0 ? void 0 : _33.icon),
                        },
                        style: styleIfNeed((_35 = (_34 = palette.color) === null || _34 === void 0 ? void 0 : _34.iterable) === null || _35 === void 0 ? void 0 : _35.iconAccent),
                    }
                },
                iterable: {
                    icon: {
                        content: {
                            line: 'I',
                        }
                    }
                },
                set: {
                    icon: {
                        content: {
                            line: 'S',
                        }
                    }
                },
                typedArray: {
                    icon: {
                        content: {
                            line: 'T',
                        }
                    }
                },
                unknown: {
                    icon: {
                        content: {
                            line: '?',
                        }
                    }
                },
            },
        },
    };
    return theme;
}
exports.smart = {
    standart: buildTheme(standartPalette),
    safeAnsi: buildTheme(safeAnsiPalette),
    richFonts: buildTheme(richFontsPalette),
};
//# sourceMappingURL=smart.js.map