"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const styleTypes_1 = require("../../styleTypes");
const simpleRenderer_1 = require("../abstract/simpleRenderer");
const os_1 = require("os");
const strictRgb_1 = require("../abstract/strictRgb");
class HtmlRenderer extends simpleRenderer_1.SimpleRenderer {
    constructor() {
        super(...arguments);
        this.eol = '<br/>' + os_1.EOL;
        this.documentBegin = '<span style="font-family: monospace; ">';
        this.documentEnd = '</span>';
    }
    styleBegin(style) {
        let styleString = '';
        if (style) {
            styleString += '<span style="';
            if (style.background) {
                const strictBackground = strictRgb_1.strictRgb(style.background);
                styleString += `background-color: rgb(${strictBackground.r}, ${strictBackground.g}, ${strictBackground.b}); `;
            }
            if (style.foreground) {
                const strictForeground = strictRgb_1.strictRgb(style.foreground);
                styleString += `color: rgb(${strictForeground.r}, ${strictForeground.g}, ${strictForeground.b}); `;
            }
            if (style.switches[styleTypes_1.StyleSwitchesEnum.Bold]) {
                styleString += 'font-weight: bold; ';
            }
            if (style.switches[styleTypes_1.StyleSwitchesEnum.Italic]) {
                styleString += 'font-style: italic; ';
            }
            if (style.switches[styleTypes_1.StyleSwitchesEnum.Underline]) {
                styleString += 'text-decoration: underline; ';
            }
            styleString += '" >';
        }
        return styleString;
    }
    styleEnd(style) {
        let styleString = '';
        if (style) {
            styleString = '</span>';
        }
        return styleString;
    }
    escapeText(text) {
        const tagsToEscape = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            ' ': '&nbsp;',
        };
        // TODO: redo such uneffective crap
        return text.replace(/[&<> ]/g, (tag) => {
            return tagsToEscape[tag] || tag;
        });
    }
    wrapDocument(document) {
        return this.documentBegin + document + this.documentEnd;
    }
}
exports.HtmlRenderer = HtmlRenderer;
//# sourceMappingURL=html.js.map