import { AnyStrictUnicodeT, StrictUnicodeLine } from './strictUnicode';
import { Style } from './style';
declare type textContainerSizeT = {
    width: {
        first: number;
        other: number;
        last: number;
    };
    heigth: number;
};
export declare abstract class TextContainerBase<T extends AnyStrictUnicodeT = AnyStrictUnicodeT> {
    style?: Style;
    abstract calcSize(): textContainerSizeT;
    abstract toString(): string;
    abstract flatten(initialStyle?: Style): FlatNonatomicTextContainer<T>;
    abstract splitToFlatFeedLines(): FlatNonatomicTextContainer<StrictUnicodeLine>[];
}
export declare type AnyTextContainer<T extends AnyStrictUnicodeT = AnyStrictUnicodeT> = AtomicTextContainer<T> | NonatomicTextContainer<T> | FlatNonatomicTextContainer<T>;
export declare class AtomicTextContainer<T extends AnyStrictUnicodeT = AnyStrictUnicodeT> extends TextContainerBase<T> {
    readonly text: T;
    style?: Style | undefined;
    readonly size: textContainerSizeT;
    constructor(text: T, style?: Style | undefined);
    precalcSize(): textContainerSizeT;
    calcSize(): textContainerSizeT;
    toString(): string;
    flatten(initialStyle?: Style): FlatNonatomicTextContainer<T>;
    splitToFlatFeedLines(): FlatNonatomicTextContainer<StrictUnicodeLine>[];
    managedWrap(maxWidth: number): Generator<AtomicTextContainer<StrictUnicodeLine>, AtomicTextContainer<StrictUnicodeLine>, number>;
}
export declare class NonatomicTextContainer<T extends AnyStrictUnicodeT = AnyStrictUnicodeT> extends TextContainerBase<T> {
    children: AnyTextContainer<T>[];
    style?: Style | undefined;
    constructor(children: AnyTextContainer<T>[], style?: Style | undefined);
    calcSize(): textContainerSizeT;
    toString(): string;
    flatten(initialStyle?: Style): FlatNonatomicTextContainer<T>;
    splitToFlatFeedLines(): FlatNonatomicTextContainer<StrictUnicodeLine>[];
    managedWrap(maxWidth: number): Generator<AnyTextContainer<StrictUnicodeLine>, AnyTextContainer<StrictUnicodeLine>, number>;
}
export declare class FlatNonatomicTextContainer<T extends AnyStrictUnicodeT = AnyStrictUnicodeT> extends NonatomicTextContainer<T> {
    children: AtomicTextContainer<T>[];
    constructor(children: AtomicTextContainer<T>[]);
    splitToFlatFeedLines(): FlatNonatomicTextContainer<StrictUnicodeLine>[];
}
export {};
//# sourceMappingURL=textContainer.d.ts.map